import yaml
import os


class gitlab_ci:
    def __init__(self, theme, docs_dir):
        self.theme = theme
        self.docs_dir = docs_dir
        self.file_exists = os.path.exists(os.getcwd() + "/.gitlab-ci.yml")
        if self.file_exists:
            with open(".gitlab-ci.yml", "r") as f:
                self.file = yaml.safe_load(f)
        else:
            self.file = {}

        return

    def _createStage(self, name):
        name = name + 'docs'
        if name in self.file['stages']:
            name = self._createStage(name)

        return name

    def addPages(self):
        if self.file_exists and self.file is not None:
            if 'pages' in self.file.keys():
                print("That's wierd..."
                      "looks like you're already deploying docs")
                return 1
            else:
                self.name = self._createStage("")
                self.file['stages'].append(self.name)
        else:
            self.file = {'stages': ['docs']}
            self.name = 'docs'

        self.file['pages'] = {'stage': f'{self.name}',
                              'image': 'python:latest',
                              'before_script': [
                                'pip install --upgrade pip',
                                'pip install Sphinx',
                                f'pip install {self.theme}'
                                  ],
                              'script':
                              f'sphinx-build -b html {self.docs_dir} public',
                              'artifacts': {'paths': ['public']}}

        return 0

    def save(self):
        with open(".gitlab-ci.yml", "w") as f:
            yaml.dump(self.file, f, sort_keys=False)

        return



from types import NoneType
import click
import os
from typing import Union
from doc_wizard import gitlab_ci
from doc_wizard import doc


def generateCi(theme, doc_dir):
    ci = gitlab_ci.gitlab_ci(theme, doc_dir)
    exists = ci.addPages()
    if exists:
        pass
    else:
        ci.save()

    return


def generateDocs(name, autodoc, doc_dir):
    os.makedirs(doc_dir)
    origin = os.getcwd()
    os.chdir(doc_dir)
    os.system('pip install Sphinx')
    if autodoc is None:
        os.system(f'sphinx-quickstart -p {name} --sep -l en')
    else:
        os.system(f'sphinx-quickstart -p {name} --ext-autodoc --sep -l en')
        with open("source/conf.py", "r") as f:
            text = f.readlines()
        text[12] = 'import os\n'
        text[13] = 'import sys\n'
        text[14] = "sys.path.insert(0, os.path.abspath('../'))\n"
        with open("source/conf.py", "w") as f:
            f.writelines(text)
    os.system('mv source/* .')
    os.chdir(origin)

    return


def parseDocs(autodoc: Union[str, NoneType], doc_dir: str, name: str):
    docs = doc.doc(autodoc, doc_dir, name)
    docs.genGettingStarted()
    docs.updateIndex()

    return


@click.command()
@click.option('-t', '--theme', default='alabaster',
              help='Specify the theme to use for the docs. \
                      Must be a pip-installable Sphinx theme. \
                      Defaults to "alabaster"')
@click.option('--no_template', is_flag=True,
              help='Set if you do not want a doc template to be \
                      generated. Set if you already have a Sphinx \
                      docs directory with documentation.')
@click.option('-d', '--doc_dir', default='docs',
              help='Directory in which doc templates will be \
                      generated. Defaults to "docs"')
@click.option('-n', '--name', default=os.path.split(os.getcwd())[1],
              help='The name of the project. It will be used in \
                      the template documentation. Defaults to \
                      the current working directory.')
@click.option('-a', '--autodoc', default=None,
              help='Directory where the modules are, relative to \
                      project base. If provided, autodoc will be \
                      enabled and Sphinx will parse the docstrings \
                      in the modules')
def parseOpts(theme: str,
              no_template: bool,
              doc_dir: str,
              name: str,
              autodoc: Union[str, NoneType]):
    generateCi(theme, doc_dir)
    if no_template:
        pass
    else:
        generateDocs(name, autodoc, doc_dir)
        parseDocs(autodoc, doc_dir, name)

    return


if __name__ == "__main__":
    parseOpts()



import os
from types import NoneType
from typing import Union


class doc:
    def __init__(self,
                 autodoc: Union[str, NoneType],
                 doc_dir: str,
                 name: str) -> None:
        self.autodoc = autodoc
        self.doc_dir = doc_dir
        self.name = name
        if autodoc is not None:
            self._genReferences()

        return

    def _genReferences(self):
        text = [".. _reference:\n",
                "\n",
                "*********\n",
                "Reference\n",
                "*********\n"]
        module_names = []
        for _, _, file in os.walk(os.getcwd()+"/"+self.autodoc):
            for file in file:
                if file.endswith('.py'):
                    module_names.append(file)
        for module in module_names:
            module = module.rsplit('.')[0]
            text.append("\n")
            text.append(f".. _{module}:\n")
            text.append("\n")
            text.append(f"{module}\n")
            decorator = ""
            for char in range(len(module)):
                decorator = decorator + "="
            text.append(f"{decorator}\n")
            text.append("\n")
            text.append(f".. automodule:: {self.autodoc.replace('/', '.')}"
                        f".{module}\n")
            text.append("   :members:\n")
        with open(f"{self.doc_dir}/source/refereces.rst", "w") as f:
            f.writelines(text)

        return

    def genGettingStarted(self):
        decorator = ""
        for char in range(len(self.name)):
            decorator = decorator + "-"
        text = [".. _getting_started:\n",
                "\n",
                "***************\n",
                "Getting Started\n",
                "***************\n",
                "\n",
                f".. _{self.name}:\n",
                "\n",
                f"{self.name}\n",
                f"{decorator}\n"]
        with open(f"{self.doc_dir}/source/getting_started.rst", "w") as f:
            f.writelines(text)

        return

    def updateIndex(self):
        with open(f"{self.doc_dir}/index.rst", "r") as f:
            text = f.readlines()
        if self.autodoc is None:
            text.insert(13, "   source/getting_started.rst\n")
            text.insert(14, "\n")
        else:
            text.insert(13, "   source/getting_started.rst\n")
            text.insert(14, "   source/refereces.rst\n")
            text.insert(15, "\n")
        with open(f"{self.doc_dir}/index.rst", "w+") as f:
            f.writelines(text)

        return



import pathlib
from setuptools import setup

# The directory containing this file
HERE = pathlib.Path(__file__).parent

# The text of the README file
README = (HERE / "README.md").read_text()

# This call to setup() does all the work
setup(
    name="doc-wizard",
    version="1.0.2",
    description="A documentation setup tool for Gitlab",
    long_description=README,
    long_description_content_type="text/markdown",
    url="https://gitlab.cern.ch/jtrielof/doc-wizard.git",
    author="Jonathan Trieloff",
    author_email="j.trieloff@cern.ch",
    license="MIT",
    classifiers=[
        "License :: OSI Approved :: MIT License",
        "Programming Language :: Python :: 3",
        "Programming Language :: Python :: 3.7",
    ],
    packages=["doc_wizard"],
    include_package_data=True,
    install_requires=['typing', 'click'],
    entry_points={
        "console_scripts": [
            "doc-wizard=doc_wizard.__main__:parseOpts",
        ]
    },
)
